@@ -1,13 +1,13 @@
 /* Copyright 2003 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: aes.h,v 1.12 2005/06/09 19:03:31 nickm Exp $ */
+/* $Id: aes.h,v 1.13 2005/12/14 20:40:39 nickm Exp $ */
 
 /* Implements a minimal interface to counter-mode AES. */
 
 #ifndef __AES_H
 #define __AES_H
-#define AES_H_ID "$Id: aes.h,v 1.12 2005/06/09 19:03:31 nickm Exp $"
+#define AES_H_ID "$Id: aes.h,v 1.13 2005/12/14 20:40:39 nickm Exp $"
 
 /**
  * \file aes.h
@@ -22,7 +22,8 @@ typedef struct aes_cnt_cipher aes_cnt_cipher_t;
 aes_cnt_cipher_t* aes_new_cipher(void);
 void aes_free_cipher(aes_cnt_cipher_t *cipher);
 void aes_set_key(aes_cnt_cipher_t *cipher, const char *key, int key_bits);
-void aes_crypt(aes_cnt_cipher_t *cipher, const char *input, size_t len, char *output);
+void aes_crypt(aes_cnt_cipher_t *cipher, const char *input, size_t len,
+               char *output);
 uint64_t aes_get_counter(aes_cnt_cipher_t *cipher);
 void aes_set_counter(aes_cnt_cipher_t *cipher, uint64_t counter);
 void aes_adjust_counter(aes_cnt_cipher_t *cipher, long delta);
