@@ -1,8 +1,9 @@
 /* Copyright 2003-2004 Roger Dingledine
    Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: container.c,v 1.53 2005/12/03 02:00:51 nickm Exp $ */
-const char container_c_id[] = "$Id: container.c,v 1.53 2005/12/03 02:00:51 nickm Exp $";
+/* $Id: container.c,v 1.57 2006/01/02 04:40:18 arma Exp $ */
+const char container_c_id[] =
+  "$Id: container.c,v 1.57 2006/01/02 04:40:18 arma Exp $";
 
 /**
  * \file container.c
@@ -78,33 +79,35 @@ smartlist_clear(smartlist_t *sl)
   sl->num_used = 0;
 }
 
-/** Append element to the end of the list. */
-void
-smartlist_add(smartlist_t *sl, void *element)
+/** Make sure that <b>sl</b> can hold at least <b>size</b> entries. */
+static INLINE void
+smartlist_ensure_capacity(smartlist_t *sl, int size)
 {
-  if (sl->num_used >= sl->capacity) {
+  if (size > sl->capacity) {
     int higher = sl->capacity * 2;
+    while (size > higher)
+      higher *= 2;
     tor_assert(higher > sl->capacity); /* detect overflow */
     sl->capacity = higher;
     sl->list = tor_realloc(sl->list, sizeof(void*)*sl->capacity);
   }
+}
+
+/** Append element to the end of the list. */
+void
+smartlist_add(smartlist_t *sl, void *element)
+{
+  smartlist_ensure_capacity(sl, sl->num_used+1);
   sl->list[sl->num_used++] = element;
 }
 
 /** Append each element from S2 to the end of S1. */
 void
-smartlist_add_all(smartlist_t *sl, const smartlist_t *s2)
+smartlist_add_all(smartlist_t *s1, const smartlist_t *s2)
 {
-  int n2 = sl->num_used + s2->num_used;
-  if (n2 > sl->capacity) {
-    int higher = sl->capacity * 2;
-    while (n2 > higher)
-      higher *= 2;
-    sl->capacity = higher;
-    sl->list = tor_realloc(sl->list, sizeof(void*)*sl->capacity);
-  }
-  memcpy(sl->list + sl->num_used, s2->list, s2->num_used*sizeof(void*));
-  sl->num_used += s2->num_used;
+  smartlist_ensure_capacity(s1, s1->num_used + s2->num_used);
+  memcpy(s1->list + s1->num_used, s2->list, s2->num_used*sizeof(void*));
+  s1->num_used += s2->num_used;
 }
 
 /** Remove all elements E from sl such that E==element.  Preserve
@@ -227,6 +230,7 @@ smartlist_del(smartlist_t *sl, int idx)
   tor_assert(idx < sl->num_used);
   sl->list[idx] = sl->list[--sl->num_used];
 }
+
 /** Remove the <b>idx</b>th element of sl; if idx is not the last element,
  * moving all subsequent elements back one space. Return the old value
  * of the <b>idx</b>th element.
@@ -241,6 +245,7 @@ smartlist_del_keeporder(smartlist_t *sl, int idx)
   if (idx < sl->num_used)
     memmove(sl->list+idx, sl->list+idx+1, sizeof(void*)*(sl->num_used-idx));
 }
+
 /** Insert the value <b>val</b> as the new <b>idx</b>th element of
  * <b>sl</b>, moving all items previously at <b>idx</b> or later
  * forward one space.
@@ -254,11 +259,7 @@ smartlist_insert(smartlist_t *sl, int idx, void *val)
   if (idx == sl->num_used) {
     smartlist_add(sl, val);
   } else {
-    /* Ensure sufficient capacity */
-    if (sl->num_used >= sl->capacity) {
-      sl->capacity *= 2;
-      sl->list = tor_realloc(sl->list, sizeof(void*)*sl->capacity);
-    }
+    smartlist_ensure_capacity(sl, sl->num_used+1);
     /* Move other elements away */
     if (idx < sl->num_used)
       memmove(sl->list + idx + 1, sl->list + idx,
@@ -437,8 +438,8 @@ smartlist_sort_strings(smartlist_t *sl)
 #define DEFINE_MAP_STRUCTS(maptype, keydecl, prefix)      \
   typedef struct prefix ## entry_t {                      \
     HT_ENTRY(prefix ## entry_t) node;                     \
-    keydecl;                                              \
     void *val;                                            \
+    keydecl;                                              \
   } prefix ## entry_t;                                    \
   struct maptype {                                        \
     HT_HEAD(prefix ## impl, prefix ## entry_t) head;      \
@@ -807,8 +808,9 @@ digestmap_iter_done(digestmap_iter_t *iter)
   return iter == NULL;
 }
 
-/** Remove all entries from <b>map</b>, and deallocate storage for those entries.
- * If free_val is provided, it is invoked on every value in <b>map</b>.
+/** Remove all entries from <b>map</b>, and deallocate storage for those
+ * entries.  If free_val is provided, it is invoked on every value in
+ * <b>map</b>.
  */
 void
 strmap_free(strmap_t *map, void (*free_val)(void*))
