@@ -1,8 +1,9 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: crypto.c,v 1.181 2005/12/08 17:38:31 nickm Exp $ */
-const char crypto_c_id[] = "$Id: crypto.c,v 1.181 2005/12/08 17:38:31 nickm Exp $";
+/* $Id: crypto.c,v 1.183 2006/01/10 21:12:06 nickm Exp $ */
+const char crypto_c_id[] =
+  "$Id: crypto.c,v 1.183 2006/01/10 21:12:06 nickm Exp $";
 
 /**
  * \file crypto.c
@@ -167,7 +168,8 @@ crypto_log_errors(int severity, const char *doing)
     func = (const char*)ERR_func_error_string(err);
     if (!msg) msg = "(null)";
     if (doing) {
-      log(severity, LD_CRYPTO, "crypto error while %s: %s (in %s:%s)", doing, msg, lib, func);
+      log(severity, LD_CRYPTO, "crypto error while %s: %s (in %s:%s)",
+          doing, msg, lib, func);
     } else {
       log(severity, LD_CRYPTO, "crypto error: %s (in %s:%s)", msg, lib, func);
     }
@@ -226,7 +228,9 @@ crypto_global_init(int useAccel)
 void
 crypto_thread_cleanup(void)
 {
+#ifndef ENABLE_0119_PARANOIA
   ERR_remove_state(0);
+#endif
 }
 
 /** Uninitialize the crypto library. Return 0 on success, -1 on failure.
@@ -235,13 +239,17 @@ int
 crypto_global_cleanup(void)
 {
   EVP_cleanup();
+#ifndef ENABLE_0119_PARANOIA
   ERR_remove_state(0);
+#endif
   ERR_free_strings();
 #ifndef NO_ENGINES
   ENGINE_cleanup();
+#ifndef ENABLE_0119_PARANOIA
   CONF_modules_unload(1);
   CRYPTO_cleanup_all_ex_data();
 #endif
+#endif
 #ifdef TOR_IS_MULTITHREADED
   if (_n_openssl_mutexes) {
     int n = _n_openssl_mutexes;
@@ -456,7 +464,8 @@ crypto_pk_read_private_key_from_string(crypto_pk_env_t *env,
  * <b>keyfile</b> into <b>env</b>.  Return 0 on success, -1 on failure.
  */
 int
-crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env, const char *keyfile)
+crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env,
+                                         const char *keyfile)
 {
   char *contents;
   int r;
@@ -487,7 +496,8 @@ crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env, const char *keyfi
  * failure, return -1.
  */
 int
-crypto_pk_write_public_key_to_string(crypto_pk_env_t *env, char **dest, size_t *len)
+crypto_pk_write_public_key_to_string(crypto_pk_env_t *env, char **dest,
+                                     size_t *len)
 {
   BUF_MEM *buf;
   BIO *b;
@@ -525,7 +535,8 @@ crypto_pk_write_public_key_to_string(crypto_pk_env_t *env, char **dest, size_t *
  * failure.
  */
 int
-crypto_pk_read_public_key_from_string(crypto_pk_env_t *env, const char *src, size_t len)
+crypto_pk_read_public_key_from_string(crypto_pk_env_t *env, const char *src,
+                                      size_t len)
 {
   BIO *b;
 
@@ -770,7 +781,8 @@ crypto_pk_public_checksig(crypto_pk_env_t *env, char *to,
   tor_assert(env);
   tor_assert(from);
   tor_assert(to);
-  r = RSA_public_decrypt(fromlen, (unsigned char*)from, (unsigned char*)to, env->key, RSA_PKCS1_PADDING);
+  r = RSA_public_decrypt(fromlen, (unsigned char*)from, (unsigned char*)to,
+                         env->key, RSA_PKCS1_PADDING);
 
   if (r<0) {
     crypto_log_errors(LOG_WARN, "checking RSA signature");
@@ -830,7 +842,8 @@ crypto_pk_private_sign(crypto_pk_env_t *env, char *to,
     /* Not a private key */
     return -1;
 
-  r = RSA_private_encrypt(fromlen, (unsigned char*)from, (unsigned char*)to, env->key, RSA_PKCS1_PADDING);
+  r = RSA_private_encrypt(fromlen, (unsigned char*)from, (unsigned char*)to,
+                          env->key, RSA_PKCS1_PADDING);
   if (r<0) {
     crypto_log_errors(LOG_WARN, "generating RSA signature");
     return -1;
@@ -950,9 +963,11 @@ crypto_pk_private_hybrid_decrypt(crypto_pk_env_t *env,
   pkeylen = crypto_pk_keysize(env);
 
   if (fromlen <= pkeylen) {
-    return crypto_pk_private_decrypt(env,to,from,fromlen,padding,warnOnFailure);
+    return crypto_pk_private_decrypt(env,to,from,fromlen,padding,
+                                     warnOnFailure);
   }
-  outlen = crypto_pk_private_decrypt(env,buf,from,pkeylen,padding,warnOnFailure);
+  outlen = crypto_pk_private_decrypt(env,buf,from,pkeylen,padding,
+                                     warnOnFailure);
   if (outlen<0) {
     log_fn(warnOnFailure?LOG_WARN:LOG_DEBUG, LD_CRYPTO,
            "Error decrypting public-key data");
@@ -1366,7 +1381,9 @@ crypto_dh_new(void)
   if (!(res->dh->g = BN_dup(dh_param_g)))
     goto err;
 
+#ifndef ENABLE_0119_PARANOIA
   res->dh->length = DH_PRIVATE_KEY_BITS;
+#endif
 
   return res;
  err:
@@ -1397,7 +1414,8 @@ crypto_dh_generate_public(crypto_dh_env_t *dh)
     return -1;
   }
   if (tor_check_dh_key(dh->dh->pub_key)<0) {
-    warn(LD_CRYPTO, "Weird! Our own DH key was invalid.  I guess once-in-the-universe chances really do happen.  Trying again.");
+    warn(LD_CRYPTO, "Weird! Our own DH key was invalid.  I guess once-in-"
+         "the-universe chances really do happen.  Trying again.");
     /* Free and clear the keys, so openssl will actually try again. */
     BN_free(dh->dh->pub_key);
     BN_free(dh->dh->priv_key);
@@ -1425,7 +1443,8 @@ crypto_dh_get_public(crypto_dh_env_t *dh, char *pubkey, size_t pubkey_len)
   bytes = BN_num_bytes(dh->dh->pub_key);
   tor_assert(bytes >= 0);
   if (pubkey_len < (size_t)bytes) {
-    warn(LD_CRYPTO, "Weird! pubkey_len (%d) was smaller than DH_BYTES (%d)", (int) pubkey_len, bytes);
+    warn(LD_CRYPTO, "Weird! pubkey_len (%d) was smaller than DH_BYTES (%d)",
+         (int) pubkey_len, bytes);
     return -1;
   }
 
@@ -1591,7 +1610,11 @@ crypto_dh_free(crypto_dh_env_t *dh)
 
 /* Use RAND_poll if openssl is 0.9.6 release or later.  (The "f" means
    "release".)  */
+#ifndef ENABLE_0119_PARANOIA
 #define USE_RAND_POLL (OPENSSL_VERSION_NUMBER >= 0x0090600fl)
+#else
+#define USE_RAND_POLL 0
+#endif
 
 /** Seed OpenSSL's random number generator with bytes from the
  * operating system.  Return 0 on success, -1 on failure.
@@ -1627,7 +1650,8 @@ crypto_seed_rng(void)
 
 #ifdef MS_WINDOWS
   if (!provider_set) {
-    if (!CryptAcquireContext(&provider, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)) {
+    if (!CryptAcquireContext(&provider, NULL, NULL, PROV_RSA_FULL,
+                             CRYPT_VERIFYCONTEXT)) {
       if (GetLastError() != NTE_BAD_KEYSET) {
         warn(LD_CRYPTO, "Can't get CryptoAPI provider [1]");
         return rand_poll_status ? 0 : -1;
@@ -1649,7 +1673,8 @@ crypto_seed_rng(void)
     n = read_all(fd, buf, sizeof(buf), 0);
     close(fd);
     if (n != sizeof(buf)) {
-      warn(LD_CRYPTO, "Error reading from entropy source (read only %d bytes).", n);
+      warn(LD_CRYPTO,
+           "Error reading from entropy source (read only %d bytes).", n);
       return -1;
     }
     RAND_seed(buf, sizeof(buf));
@@ -1729,7 +1754,8 @@ base64_encode(char *dest, size_t destlen, const char *src, size_t srclen)
     return -1;
 
   EVP_EncodeInit(&ctx);
-  EVP_EncodeUpdate(&ctx, (unsigned char*)dest, &len, (unsigned char*)src, srclen);
+  EVP_EncodeUpdate(&ctx, (unsigned char*)dest, &len,
+                   (unsigned char*)src, srclen);
   EVP_EncodeFinal(&ctx, (unsigned char*)(dest+len), &ret);
   ret += len;
   return ret;
@@ -1758,7 +1784,8 @@ base64_decode(char *dest, size_t destlen, const char *src, size_t srclen)
     return -1;
 
   EVP_DecodeInit(&ctx);
-  EVP_DecodeUpdate(&ctx, (unsigned char*)dest, &len, (unsigned char*)src, srclen);
+  EVP_DecodeUpdate(&ctx, (unsigned char*)dest, &len,
+                   (unsigned char*)src, srclen);
   EVP_DecodeFinal(&ctx, (unsigned char*)dest, &ret);
   ret += len;
   return ret;
