@@ -1,8 +1,8 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: log.c,v 1.109 2005/11/30 06:45:30 arma Exp $ */
-const char log_c_id[] = "$Id: log.c,v 1.109 2005/11/30 06:45:30 arma Exp $";
+/* $Id: log.c,v 1.110 2005/12/14 20:40:39 nickm Exp $ */
+const char log_c_id[] = "$Id: log.c,v 1.110 2005/12/14 20:40:39 nickm Exp $";
 
 /**
  * \file log.c
@@ -364,7 +364,8 @@ close_log(logfile_t *victim)
 /** Add a log handler to send all messages of severity <b>loglevel</b>
  * or higher to <b>stream</b>. */
 void
-add_stream_log(int loglevelMin, int loglevelMax, const char *name, FILE *stream)
+add_stream_log(int loglevelMin, int loglevelMax,
+               const char *name, FILE *stream)
 {
   logfile_t *lf;
   lf = tor_malloc_zero(sizeof(logfile_t));
