@@ -1,8 +1,8 @@
 /* Copyright 2003 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.c,v 1.239 2005/12/09 02:45:33 arma Exp $ */
-const char util_c_id[] = "$Id: util.c,v 1.239 2005/12/09 02:45:33 arma Exp $";
+/* $Id: util.c,v 1.243 2006/01/10 20:00:20 nickm Exp $ */
+const char util_c_id[] = "$Id: util.c,v 1.243 2006/01/10 20:00:20 nickm Exp $";
 
 /**
  * \file util.c
@@ -101,10 +101,12 @@ const char util_c_id[] = "$Id: util.c,v 1.239 2005/12/09 02:45:33 arma Exp $";
 #else
  #define dmalloc_strdup(file, line, string, xalloc_b) strdup(string)
 
- #define dmalloc_malloc(file, line, size, func_id, alignment, xalloc_b) malloc(size)
+ #define dmalloc_malloc(file, line, size, func_id, alignment, xalloc_b) \
+     malloc(size)
  #define DMALLOC_FUNC_MALLOC 0
 
- #define dmalloc_realloc(file, line, old_pnt, new_size, func_id, xalloc_b) realloc((old_pnt), (new_size))
+ #define dmalloc_realloc(file, line, old_pnt, new_size, func_id, xalloc_b) \
+     realloc((old_pnt), (new_size))
  #define DMALLOC_FUNC_REALLOC 0
  #define DMALLOC_FN_ARGS
 #endif
@@ -644,9 +646,10 @@ tor_timegm(struct tm *tm)
   unsigned long year, days, hours, minutes;
   int i;
   year = tm->tm_year + 1900;
-  tor_assert(year >= 1970);
-  tor_assert(tm->tm_mon >= 0);
-  tor_assert(tm->tm_mon <= 11);
+  if (year < 1970 || tm->tm_mon < 0 || tm->tm_mon > 11) {
+    warn(LD_BUG, "Out-of-range argument to tor_timegm");
+    return -1;
+  }
   days = 365 * (year-1970) + n_leapdays(1970,year);
   for (i = 0; i < tm->tm_mon; ++i)
     days += days_per_month[i];
@@ -712,9 +715,14 @@ parse_rfc1123_time(const char *buf, time_t *t)
     warn(LD_GENERAL, "Got invalid RFC1123 time \"%s\"", buf);
     return -1;
   }
-
   tm.tm_mon = m;
+
+  if (tm.tm_year < 1970) {
+    warn(LD_GENERAL, "Got invalid RFC1123 time \"%s\". (Before 1970)", buf);
+    return -1;
+  }
   tm.tm_year -= 1900;
+
   *t = tor_timegm(&tm);
   return 0;
 }
@@ -758,6 +766,10 @@ parse_iso_time(const char *cp, time_t *t)
   st_tm.tm_min = minute;
   st_tm.tm_sec = second;
 #endif
+  if (st_tm.tm_year < 70) {
+    warn(LD_GENERAL, "Got invalid ISO time \"%s\". (Before 1970)", cp);
+    return -1;
+  }
   *t = tor_timegm(&st_tm);
   return 0;
 }
@@ -921,7 +933,8 @@ check_private_dir(const char *dirname, cpd_check_t check)
 
     pw = getpwuid(st.st_uid);
 
-    log(LOG_WARN, LD_FS, "%s is not owned by this user (%s, %d) but by %s (%d). Perhaps you are running Tor as the wrong user?",
+    log(LOG_WARN, LD_FS, "%s is not owned by this user (%s, %d) but by "
+        "%s (%d). Perhaps you are running Tor as the wrong user?",
                          dirname, process_ownername, (int)getuid(),
                          pw ? pw->pw_name : "<unknown>", (int)st.st_uid);
 
@@ -942,8 +955,8 @@ check_private_dir(const char *dirname, cpd_check_t check)
   return 0;
 }
 
-/** Create a file named <b>fname</b> with the contents <b>str</b>.  Overwrite the
- * previous <b>fname</b> if possible.  Return 0 on success, -1 on failure.
+/** Create a file named <b>fname</b> with the contents <b>str</b>.  Overwrite
+ * the previous <b>fname</b> if possible.  Return 0 on success, -1 on failure.
  *
  * This function replaces the old file atomically, if possible.
  */
@@ -989,18 +1002,21 @@ write_chunks_to_file_impl(const char *fname, const smartlist_t *chunks,
   {
     result = write_all(fd, chunk->bytes, chunk->len, 0);
     if (result < 0 || (size_t)result != chunk->len) {
-      log(LOG_WARN, LD_FS, "Error writing to \"%s\": %s", tempname, strerror(errno));
+      log(LOG_WARN, LD_FS, "Error writing to \"%s\": %s", tempname,
+          strerror(errno));
       close(fd);
       goto err;
     }
   });
   if (close(fd)) {
-    log(LOG_WARN, LD_FS, "Error flushing to \"%s\": %s", tempname, strerror(errno));
+    log(LOG_WARN, LD_FS, "Error flushing to \"%s\": %s", tempname,
+        strerror(errno));
     goto err;
   }
   if (!(open_flags & O_APPEND)) {
     if (replace_file(tempname, fname)) {
-      log(LOG_WARN, LD_FS, "Error replacing \"%s\": %s", fname, strerror(errno));
+      log(LOG_WARN, LD_FS, "Error replacing \"%s\": %s", fname,
+          strerror(errno));
       goto err;
     }
   }
@@ -1108,8 +1124,9 @@ read_file_to_str(const char *filename, int bin)
   }
 #ifdef MS_WINDOWS
   if (!bin && strchr(string, '\r')) {
-    debug(LD_FS, "We didn't convert CRLF to LF as well as we hoped when reading %s. Coping.",
-           filename);
+    debug(LD_FS, "We didn't convert CRLF to LF as well as we hoped "
+          "when reading %s. Coping.",
+          filename);
     tor_strstrip(string, "\r");
   }
 #endif
@@ -1203,7 +1220,8 @@ expand_filename(const char *filename)
     if (filename[1] == '/' || filename[1] == '\0') {
       home = getenv("HOME");
       if (!home) {
-        warn(LD_CONFIG, "Couldn't find $HOME environment variable while expanding %s", filename);
+        warn(LD_CONFIG, "Couldn't find $HOME environment variable while "
+             "expanding %s", filename);
         return NULL;
       }
       home = tor_strdup(home);
@@ -1296,6 +1314,17 @@ tor_listdir(const char *dirname)
   return result;
 }
 
+/** Return true iff <b>filename</b> is a relative path.  (XXXX doesn't work on
+ * windows.) */
+int
+path_is_relative(const char *filename)
+{
+  if (filename && filename[0] == '/')
+    return 0;
+  else
+    return 1;
+}
+
 /* =====
  * Net helpers
  * ===== */
@@ -1430,7 +1459,8 @@ parse_addr_and_port_range(const char *s, uint32_t *addr_out,
   } else if (tor_inet_aton(address, &in) != 0) {
     *addr_out = ntohl(in.s_addr);
   } else {
-    warn(LD_GENERAL, "Malformed IP \"%s\" in address pattern; rejecting.",address);
+    warn(LD_GENERAL, "Malformed IP \"%s\" in address pattern; rejecting.",
+         address);
     goto err;
   }
 
@@ -1445,7 +1475,8 @@ parse_addr_and_port_range(const char *s, uint32_t *addr_out,
     if (!*endptr) {
       /* strtol handled the whole mask. */
       if (bits < 0 || bits > 32) {
-        warn(LD_GENERAL, "Bad number of mask bits on address range; rejecting.");
+        warn(LD_GENERAL,
+             "Bad number of mask bits on address range; rejecting.");
         goto err;
       }
       *mask_out = ~((1<<(32-bits))-1);
