@@ -1,8 +1,9 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c,v 1.277 2005/12/10 09:36:25 arma Exp $ */
-const char dirserv_c_id[] = "$Id: dirserv.c,v 1.277 2005/12/10 09:36:25 arma Exp $";
+/* $Id: dirserv.c,v 1.291 2006/01/11 03:59:55 nickm Exp $ */
+const char dirserv_c_id[] =
+  "$Id: dirserv.c,v 1.291 2006/01/11 03:59:55 nickm Exp $";
 
 #include "or.h"
 
@@ -16,6 +17,9 @@ const char dirserv_c_id[] = "$Id: dirserv.c,v 1.277 2005/12/10 09:36:25 arma Exp
 #define ROUTER_ALLOW_SKEW (60*60*12) /* 12 hours */
 /** How many seconds do we wait before regenerating the directory? */
 #define DIR_REGEN_SLACK_TIME 30
+/** If we're a cache, keep this many networkstatuses around from non-trusted
+ * directory authorities. */
+#define MAX_UNTRUSTED_NETWORKSTATUSES 16
 
 extern long stats_n_seconds_working;
 
@@ -35,7 +39,8 @@ static void directory_remove_invalid(void);
 static int dirserv_regenerate_directory(void);
 static char *format_versions_list(config_line_t *ln);
 /* Should be static; exposed for testing */
-int add_fingerprint_to_dir(const char *nickname, const char *fp, smartlist_t *list);
+int add_fingerprint_to_dir(const char *nickname, const char *fp,
+                           smartlist_t *list);
 static int router_is_general_exit(routerinfo_t *ri);
 static router_status_t dirserv_router_get_status(const routerinfo_t *router,
                                                  const char **msg);
@@ -92,7 +97,8 @@ typedef struct fingerprint_entry_t {
                    * always be rejected); or the string "!invalid" (if this
                    * fingerprint should be accepted but never marked as
                    * valid. */
-  char *fingerprint; /**< Stored as HEX_DIGEST_LEN characters, followed by a NUL */
+  char *fingerprint; /**< Stored as HEX_DIGEST_LEN characters, followed by a
+                      * NUL */
 } fingerprint_entry_t;
 
 /** List of nickname-\>identity fingerprint mappings for all the routers
@@ -208,7 +214,8 @@ dirserv_parse_fingerprint_file(const char *fname)
              DEFAULT_CLIENT_NICKNAME);
       continue;
     }
-    if (add_fingerprint_to_dir(nickname, fingerprint, fingerprint_list_new) != 0)
+    if (add_fingerprint_to_dir(nickname, fingerprint, fingerprint_list_new)
+        != 0)
       notice(LD_CONFIG, "Duplicate nickname '%s'.", nickname);
   }
 
@@ -276,11 +283,13 @@ dirserv_get_status_impl(const char *fp, const char *nickname,
                         const char **msg, int should_log)
 {
   fingerprint_entry_t *nn_ent = NULL, *fp_ent = NULL;
+  int reject_unlisted = get_options()->AuthDirRejectUnlisted;
   if (!fingerprint_list)
     fingerprint_list = smartlist_create();
 
   if (should_log)
-    debug(LD_DIRSERV, "%d fingerprints known.", smartlist_len(fingerprint_list));
+    debug(LD_DIRSERV, "%d fingerprints known.",
+          smartlist_len(fingerprint_list));
   SMARTLIST_FOREACH(fingerprint_list, fingerprint_entry_t *, ent,
   {
     if (!strcasecmp(fp,ent->fingerprint))
@@ -296,7 +305,7 @@ dirserv_get_status_impl(const char *fp, const char *nickname,
       return FP_REJECT;
     } else if (!strcasecmp(fp_ent->nickname, "!invalid")) {
       if (msg)
-        *msg = "Fingerprint is marged invalid";
+        *msg = "Fingerprint is marked invalid";
       return FP_INVALID;
     }
   }
@@ -309,7 +318,7 @@ dirserv_get_status_impl(const char *fp, const char *nickname,
 
     if (rej == ADDR_POLICY_PROBABLY_REJECTED || rej == ADDR_POLICY_REJECTED) {
       if (should_log)
-        info(LD_DIRSERV, "Rejecting '%s' because of address %s",
+        info(LD_DIRSERV, "Rejecting '%s' because of address '%s'",
              nickname, address);
       if (msg)
         *msg = "Authdir is rejecting routers in this range.";
@@ -317,17 +326,16 @@ dirserv_get_status_impl(const char *fp, const char *nickname,
     }
     if (inv == ADDR_POLICY_PROBABLY_REJECTED || inv == ADDR_POLICY_REJECTED) {
       if (should_log)
-        info(LD_DIRSERV, "Not marking '%s' valid because of address %s",
+        info(LD_DIRSERV, "Not marking '%s' valid because of address '%s'",
              nickname, address);
       return FP_INVALID;
     }
+    if (should_log)
+      debug(LD_DIRSERV,"No fingerprint found for '%s'",nickname);
     if (!platform || tor_version_as_new_as(platform,"0.1.0.2-rc"))
-      return FP_VALID;
+      return reject_unlisted ? FP_REJECT : FP_VALID;
     else
       return FP_INVALID;
-    if (should_log)
-      info(LD_DIRSERV,"No fingerprint found for '%s'",nickname);
-    return 0;
   }
   if (0==strcasecmp(nn_ent->fingerprint, fp)) {
     if (should_log)
@@ -342,7 +350,8 @@ dirserv_get_status_impl(const char *fp, const char *nickname,
            contact ? contact : "",
            platform ? platform : "");
     if (msg)
-      *msg = "Rejected: There is already a verified server with this nickname and a different fingerprint.";
+      *msg = "Rejected: There is already a verified server with this nickname "
+        "and a different fingerprint.";
     return FP_REJECT; /* Wrong fingerprint. */
   }
 }
@@ -411,17 +420,21 @@ dirserv_router_has_valid_address(routerinfo_t *ri)
 }
 
 /** Check whether we, as a directory server, want to accept <b>ri</b>.  If so,
- * return 0, and set its is_valid,named,running fields.  Otherwise, return -1.
+ * set its is_valid,named,running fields and return 0.  Otherwise, return -1.
  *
  * If the router is rejected, set *<b>msg</b> to an explanation of why.
+ *
+ * If <b>complain</b> then explain at log-level 'notice' why we refused
+ * a descriptor; else explain at log-level 'info'.
  */
 int
-authdir_wants_to_reject_router(routerinfo_t *ri,
-                               const char **msg)
+authdir_wants_to_reject_router(routerinfo_t *ri, const char **msg,
+                               int complain)
 {
   /* Okay.  Now check whether the fingerprint is recognized. */
   router_status_t status = dirserv_router_get_status(ri, msg);
   time_t now;
+  int severity = complain ? LOG_NOTICE : LOG_INFO;
   tor_assert(msg);
   if (status == FP_REJECT)
     return -1; /* msg is already set. */
@@ -429,27 +442,32 @@ authdir_wants_to_reject_router(routerinfo_t *ri,
   /* Is there too much clock skew? */
   now = time(NULL);
   if (ri->cache_info.published_on > now+ROUTER_ALLOW_SKEW) {
-    notice(LD_DIRSERV, "Publication time for nickname '%s' is too far "
-           "(%d minutes) in the future; possible clock skew. Not adding "
+    log_fn(severity, LD_DIRSERV, "Publication time for nickname '%s' is too "
+           "far (%d minutes) in the future; possible clock skew. Not adding "
            "(ContactInfo '%s', platform '%s').",
            ri->nickname, (int)((ri->cache_info.published_on-now)/60),
            ri->contact_info ? ri->contact_info : "",
            ri->platform ? ri->platform : "");
-    *msg = "Rejected: Your clock is set too far in the future, or your timezone is not correct.";
+    *msg = "Rejected: Your clock is set too far in the future, or your "
+      "timezone is not correct.";
     return -1;
   }
   if (ri->cache_info.published_on < now-ROUTER_MAX_AGE) {
-    notice(LD_DIRSERV,
+    log_fn(severity, LD_DIRSERV,
            "Publication time for router with nickname '%s' is too far "
-           "(%d minutes) in the past. Not adding (ContactInfo '%s', platform '%s').",
+           "(%d minutes) in the past. Not adding (ContactInfo '%s', "
+           "platform '%s').",
            ri->nickname, (int)((now-ri->cache_info.published_on)/60),
            ri->contact_info ? ri->contact_info : "",
            ri->platform ? ri->platform : "");
-    *msg = "Rejected: Server is expired, or your clock is too far in the past, or your timezone is not correct.";
+    *msg = "Rejected: Server is expired, or your clock is too far in the past,"
+      " or your timezone is not correct.";
     return -1;
   }
   if (dirserv_router_has_valid_address(ri) < 0) {
-    notice(LD_DIRSERV, "Router with nickname '%s' has invalid address '%s'. Not adding (ContactInfo '%s', platform '%s').",
+    log_fn(severity, LD_DIRSERV,
+           "Router with nickname '%s' has invalid address '%s'. "
+           "Not adding (ContactInfo '%s', platform '%s').",
            ri->nickname, ri->address,
            ri->contact_info ? ri->contact_info : "",
            ri->platform ? ri->platform : "");
@@ -506,16 +524,18 @@ dirserv_add_descriptor(const char *desc, const char **msg)
    * another authority has, so we all converge on the same one.) */
   ri_old = router_get_by_digest(ri->cache_info.identity_digest);
   if (ri_old && ri_old->cache_info.published_on < ri->cache_info.published_on
-      && router_differences_are_cosmetic(ri_old, ri)) {
+      && router_differences_are_cosmetic(ri_old, ri)
+      && !router_is_me(ri)) {
     info(LD_DIRSERV,
          "Not replacing descriptor from '%s'; differences are cosmetic.",
          ri->nickname);
-    *msg = "Not replacing router descriptor; no information has changed since the last one with this identity.";
+    *msg = "Not replacing router descriptor; no information has changed since "
+      "the last one with this identity.";
     routerinfo_free(ri);
     control_event_or_authdir_new_descriptor("DROPPED", desc, *msg);
     return 0;
   }
-  if ((r = router_add_to_routerlist(ri, msg, 0))<0) {
+  if ((r = router_add_to_routerlist(ri, msg, 0, 0))<0) {
     if (r < -1) /* unless the routerinfo was fine, just out-of-date */
       control_event_or_authdir_new_descriptor("REJECTED", desc, *msg);
     return r == -1 ? 0 : -1;
@@ -666,8 +686,8 @@ list_single_server_status(routerinfo_t *desc, int is_live)
   return tor_strdup(buf);
 }
 
-#define REACHABLE_TIMEOUT (60*60) /* an hour */
-/* Make sure this is 3 times the value of get_dir_fetch_period() */
+#define REACHABLE_TIMEOUT (90*60) /* ninety minutes */
+/* Make sure this is at least 3 times the value of get_dir_fetch_period() */
 
 /** Treat a router as alive if
  *    - It's me, and I'm not hibernating.
@@ -678,7 +698,8 @@ dirserv_thinks_router_is_reachable(routerinfo_t *router, time_t now)
   connection_t *conn;
   if (router_is_me(router) && !we_are_hibernating())
     return 1;
-  conn = connection_or_get_by_identity_digest(router->cache_info.identity_digest);
+  conn = connection_or_get_by_identity_digest(
+      router->cache_info.identity_digest);
   if (conn && conn->state == OR_CONN_STATE_OPEN)
     return get_options()->AssumeReachable ||
            now < router->last_reachable + REACHABLE_TIMEOUT;
@@ -689,12 +710,14 @@ dirserv_thinks_router_is_reachable(routerinfo_t *router, time_t now)
  * <b>router</b>'s reachability and its operator should be notified.
  */
 int
-dirserv_thinks_router_is_blatantly_unreachable(routerinfo_t *router, time_t now)
+dirserv_thinks_router_is_blatantly_unreachable(routerinfo_t *router,
+                                               time_t now)
 {
   connection_t *conn;
   if (router->is_hibernating)
     return 0;
-  conn = connection_or_get_by_identity_digest(router->cache_info.identity_digest);
+  conn = connection_or_get_by_identity_digest(
+    router->cache_info.identity_digest);
   if (conn && conn->state == OR_CONN_STATE_OPEN &&
       now >= router->last_reachable + 2*REACHABLE_TIMEOUT &&
       router->testing_since &&
@@ -755,8 +778,8 @@ _compare_tor_version_str_ptr(const void **_a, const void **_b)
     return -1;
   if (ca && !cb)
     return 1;
-  /* If neither parses, compare strings.  Also, the directory server admin needs
-  ** to be smacked upside the head.  But Tor is tolerant and gentle. */
+  /* If neither parses, compare strings.  Also, the directory server admin
+  ** needs to be smacked upside the head.  But Tor is tolerant and gentle. */
   return strcmp(a,b);
 }
 
@@ -815,7 +838,8 @@ dirserv_dump_directory_to_string(char **dir_out,
     return -1;
   }
 
-  recommended_versions = format_versions_list(get_options()->RecommendedVersions);
+  recommended_versions =
+    format_versions_list(get_options()->RecommendedVersions);
 
   published_on = time(NULL);
   format_iso_time(published, published_on);
@@ -964,28 +988,58 @@ dirserv_set_cached_directory(const char *directory, time_t published,
  * the cache.
  */
 void
-dirserv_set_cached_networkstatus_v2(const char *directory,
+dirserv_set_cached_networkstatus_v2(const char *networkstatus,
                                     const char *identity,
                                     time_t published)
 {
   cached_dir_t *d;
+  smartlist_t *trusted_dirs;
   if (!cached_v2_networkstatus)
     cached_v2_networkstatus = digestmap_new();
 
   if (!(d = digestmap_get(cached_v2_networkstatus, identity))) {
-    if (!directory)
+    if (!networkstatus)
       return;
     d = tor_malloc_zero(sizeof(cached_dir_t));
     digestmap_set(cached_v2_networkstatus, identity, d);
   }
 
   tor_assert(d);
-  if (directory) {
-    set_cached_dir(d, tor_strdup(directory), published);
+  if (networkstatus) {
+    if (published > d->published) {
+      set_cached_dir(d, tor_strdup(networkstatus), published);
+    }
   } else {
     free_cached_dir(d);
     digestmap_remove(cached_v2_networkstatus, identity);
   }
+
+  router_get_trusted_dir_servers(&trusted_dirs);
+  if (digestmap_size(cached_v2_networkstatus) >
+      smartlist_len(trusted_dirs) + MAX_UNTRUSTED_NETWORKSTATUSES) {
+    /* We need to remove the oldest untrusted networkstatus. */
+    const char *oldest = NULL;
+    time_t oldest_published = TIME_MAX;
+    digestmap_iter_t *iter;
+
+    for (iter = digestmap_iter_init(cached_v2_networkstatus);
+         !digestmap_iter_done(iter);
+         iter = digestmap_iter_next(cached_v2_networkstatus, iter)) {
+      const char *ident;
+      void *val;
+      digestmap_iter_get(iter, &ident, &val);
+      d = val;
+      if (d->published < oldest_published &&
+          !router_get_trusteddirserver_by_digest(ident)) {
+        oldest = ident;
+        oldest_published = d->published;
+      }
+    }
+    tor_assert(oldest);
+    d = digestmap_remove(cached_v2_networkstatus, oldest);
+    if (d)
+      free_cached_dir(d);
+  }
 }
 
 /** Helper: If we're an authority for the right directory version (the
@@ -1132,7 +1186,8 @@ generate_runningrouters(void)
                "router-status %s\n"
                "dir-signing-key\n%s"
                "directory-signature %s\n",
-               published, router_status, identity_pkey, get_options()->Nickname);
+               published, router_status, identity_pkey,
+               get_options()->Nickname);
   tor_free(router_status);
   tor_free(identity_pkey);
   if (router_get_runningrouters_hash(s,digest)) {
@@ -1203,6 +1258,61 @@ should_generate_v2_networkstatus(void)
     the_v2_networkstatus_is_dirty + DIR_REGEN_SLACK_TIME < time(NULL);
 }
 
+long stable_uptime = 0; /* start at a safe value */
+
+/** Return 1 if <b>router</b> is not suitable for these parameters, else 0.
+ * If <b>need_uptime</b> is non-zero, we require a minimum uptime.
+ * If <b>need_capacity</b> is non-zero, we require a minimum advertised
+ * bandwidth.
+ */
+static int
+dirserv_thinks_router_is_unreliable(routerinfo_t *router,
+                                    int need_uptime, int need_capacity)
+{
+  if (need_uptime && router->uptime < stable_uptime)
+    return 1;
+  if (need_capacity &&
+      router->bandwidthcapacity < ROUTER_REQUIRED_MIN_BANDWIDTH)
+    return 1;
+  return 0;
+}
+
+static int
+_compare_longs(const void **a, const void **b)
+{
+  long first = **(long **)a, second = **(long **)b;
+  if (first < second) return -1;
+  if (first > second) return 1;
+  return 0;
+}
+
+/** Look through the routerlist, and assign the median uptime
+ * of running verified servers to stable_uptime. */
+static void
+dirserv_compute_stable_uptime(routerlist_t *rl)
+{
+  smartlist_t *uptimes = smartlist_create();
+  long *up;
+
+  SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ri, {
+    if (ri->is_running && ri->is_verified) {
+      up = tor_malloc(sizeof(long));
+      *up = ri->uptime;
+      smartlist_add(uptimes, up);
+    }
+  });
+
+  smartlist_sort(uptimes, _compare_longs);
+
+  stable_uptime = *(long *)smartlist_get(uptimes,
+                                         smartlist_len(uptimes)/2);
+
+  info(LD_DIRSERV, "Uptime cutoff is %ld seconds.", stable_uptime);
+
+  SMARTLIST_FOREACH(uptimes, long *, up, tor_free(up));
+  smartlist_free(uptimes);
+}
+
 /** For authoritative directories only: replace the contents of
  * <b>the_v2_networkstatus</b> with a newly generated network status
  * object. */
@@ -1264,7 +1374,8 @@ generate_v2_networkstatus(void)
   if (!contact)
     contact = "(none)";
 
-  len = 2048+strlen(client_versions)+strlen(server_versions)+identity_pkey_len*2;
+  len = 2048+strlen(client_versions)+strlen(server_versions);
+  len += identity_pkey_len*2;
   len += (RS_ENTRY_LEN)*smartlist_len(rl->routers);
 
   status = tor_malloc(len);
@@ -1290,21 +1401,31 @@ generate_v2_networkstatus(void)
   outp = status + strlen(status);
   endp = status + len;
 
+  /* precompute this part, since we need it to decide what "stable"
+   * means. */
+  SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ri, {
+    ri->is_running = dirserv_thinks_router_is_reachable(ri, now);
+  });
+
+  dirserv_compute_stable_uptime(rl);
+
   SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ri, {
       int f_exit = router_is_general_exit(ri);
-      int f_stable = !router_is_unreliable(ri, 1, 0);
-      int f_fast = !router_is_unreliable(ri, 0, 1);
-      int f_running;
+      int f_stable = ri->is_stable =
+                     !dirserv_thinks_router_is_unreliable(ri, 1, 0);
+      int f_fast = ri->is_fast =
+                   !dirserv_thinks_router_is_unreliable(ri, 0, 1);
+      int f_running = ri->is_running; /* computed above */
       int f_authority = router_digest_is_trusted_dir(
                                       ri->cache_info.identity_digest);
       int f_named = naming && ri->is_named;
       int f_valid = ri->is_verified;
+      /* 0.1.1.9-alpha is the first version to support fetch by descriptor
+       * hash. */
+      int f_v2_dir = ri->dir_port &&
+        tor_version_as_new_as(ri->platform,"0.1.1.9-alpha");
       char identity64[BASE64_DIGEST_LEN+1];
       char digest64[BASE64_DIGEST_LEN+1];
-      if (options->AuthoritativeDir) {
-        ri->is_running = dirserv_thinks_router_is_reachable(ri, now);
-      }
-      f_running = ri->is_running;
 
       format_iso_time(published, ri->cache_info.published_on);
 
@@ -1316,7 +1437,7 @@ generate_v2_networkstatus(void)
 
       if (tor_snprintf(outp, endp-outp,
                        "r %s %s %s %s %s %d %d\n"
-                       "s%s%s%s%s%s%s%s\n",
+                       "s%s%s%s%s%s%s%s%s\n",
                        ri->nickname,
                        identity64,
                        digest64,
@@ -1330,7 +1451,8 @@ generate_v2_networkstatus(void)
                        f_named?" Named":"",
                        f_stable?" Stable":"",
                        f_running?" Running":"",
-                       f_valid?" Valid":"")<0) {
+                       f_valid?" Valid":"",
+                       f_v2_dir?" V2Dir":"")<0) {
         warn(LD_BUG, "Unable to print router status.");
         goto done;
       }
@@ -1428,7 +1550,8 @@ dirserv_get_networkstatus_v2(smartlist_t *result,
           } else {
             char hexbuf[HEX_DIGEST_LEN+1];
             base16_encode(hexbuf, sizeof(hexbuf), cp, DIGEST_LEN);
-            info(LD_DIRSERV, "Don't know about any network status with fingerprint '%s'", hexbuf);
+            info(LD_DIRSERV, "Don't know about any network status with "
+                 "fingerprint '%s'", hexbuf);
           }
           tor_free(cp);
         });
@@ -1480,6 +1603,7 @@ dirserv_get_routerdescs(smartlist_t *descs_out, const char *key,
     smartlist_free(digests);
   } else if (!strcmpstart(key, "/tor/server/fp/")) {
     smartlist_t *digests = smartlist_create();
+    time_t cutoff = time(NULL) - ROUTER_MAX_AGE;
     key += strlen("/tor/server/fp/");
     dir_split_resource_into_fingerprints(key, digests, NULL, 1);
     SMARTLIST_FOREACH(digests, const char *, d,
@@ -1488,7 +1612,11 @@ dirserv_get_routerdescs(smartlist_t *descs_out, const char *key,
            smartlist_add(descs_out, &(router_get_my_routerinfo()->cache_info));
          } else {
            routerinfo_t *ri = router_get_by_digest(d);
-           if (ri)
+           /* Don't actually serve a descriptor that everyone will think is
+            * expired.  This is an (ugly) workaround to keep buggy 0.1.1.10
+            * Tors from downloading descriptors that they will throw away.
+            */
+           if (ri && ri->cache_info.published_on > cutoff)
              smartlist_add(descs_out, &(ri->cache_info));
          }
        });
@@ -1539,12 +1667,14 @@ dirserv_orconn_tls_done(const char *address,
     if (!ri->is_verified) {
       /* We have a router at the same address! */
       if (strcasecmp(ri->nickname, nickname_rcvd)) {
-        notice(LD_DIRSERV, "Dropping descriptor: nickname '%s' does not match nickname '%s' in cert from %s:%d",
+        notice(LD_DIRSERV, "Dropping descriptor: nickname '%s' does not match "
+               "nickname '%s' in cert from %s:%d",
                ri->nickname, nickname_rcvd, address, or_port);
         drop = 1;
       } else if (memcmp(ri->cache_info.identity_digest, digest_rcvd,
                         DIGEST_LEN)) {
-        notice(LD_DIRSERV, "Dropping descriptor: identity key does not match key in cert from %s:%d",
+        notice(LD_DIRSERV, "Dropping descriptor: identity key does not match "
+               "key in cert from %s:%d",
                address, or_port);
         drop = 1;
       }
