@@ -1,7 +1,8 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rephist.c,v 1.72 2005/11/25 08:08:56 arma Exp $ */
-const char rephist_c_id[] = "$Id: rephist.c,v 1.72 2005/11/25 08:08:56 arma Exp $";
+/* $Id: rephist.c,v 1.80 2006/01/11 04:04:42 nickm Exp $ */
+const char rephist_c_id[] =
+  "$Id: rephist.c,v 1.80 2006/01/11 04:04:42 nickm Exp $";
 
 /**
  * \file rephist.c
@@ -291,7 +292,8 @@ rep_hist_dump_stats(time_t now, int severity)
 
   log(severity, LD_GENERAL, "--------------- Dumping history information:");
 
-  for (orhist_it = digestmap_iter_init(history_map); !digestmap_iter_done(orhist_it);
+  for (orhist_it = digestmap_iter_init(history_map);
+       !digestmap_iter_done(orhist_it);
        orhist_it = digestmap_iter_next(history_map,orhist_it)) {
     digestmap_iter_get(orhist_it, &digest1, &or_history_p);
     or_history = (or_history_t*) or_history_p;
@@ -320,7 +322,8 @@ rep_hist_dump_stats(time_t now, int severity)
       len = strlen(buffer);
       for (lhist_it = digestmap_iter_init(or_history->link_history_map);
            !digestmap_iter_done(lhist_it);
-           lhist_it = digestmap_iter_next(or_history->link_history_map, lhist_it)) {
+           lhist_it = digestmap_iter_next(or_history->link_history_map,
+                                          lhist_it)) {
         digestmap_iter_get(lhist_it, &digest2, &link_history_p);
         if ((r = router_get_by_digest(digest2)))
           name2 = r->nickname;
@@ -367,7 +370,8 @@ rep_history_clean(time_t before)
       digestmap_iter_get(lhist_it, &d2, &link_history_p);
       link_history = link_history_p;
       if (link_history->changed < before) {
-        lhist_it = digestmap_iter_next_rmv(or_history->link_history_map,lhist_it);
+        lhist_it = digestmap_iter_next_rmv(or_history->link_history_map,
+                                           lhist_it);
         rephist_total_alloc -= sizeof(link_history_t);
         tor_free(link_history);
         continue;
@@ -390,13 +394,15 @@ rep_history_clean(time_t before)
 typedef struct bw_array_t {
   /** Observation array: Total number of bytes transferred in each of the last
    * NUM_SECS_ROLLING_MEASURE seconds. This is used as a circular array. */
-  int obs[NUM_SECS_ROLLING_MEASURE];
+  uint64_t obs[NUM_SECS_ROLLING_MEASURE];
   int cur_obs_idx; /**< Current position in obs. */
   time_t cur_obs_time; /**< Time represented in obs[cur_obs_idx] */
-  int total_obs; /**< Total for all members of obs except obs[cur_obs_idx] */
-  int max_total; /**< Largest value that total_obs has taken on in the current
-                  * period. */
-  uint64_t total_in_period; /**< Total bytes transferred in the current period. */
+  uint64_t total_obs; /**< Total for all members of obs except
+                       * obs[cur_obs_idx] */
+  uint64_t max_total; /**< Largest value that total_obs has taken on in the
+                       * current period. */
+  uint64_t total_in_period; /**< Total bytes transferred in the current
+                             * period. */
 
   /** When does the next period begin? */
   time_t next_period;
@@ -408,7 +414,7 @@ typedef struct bw_array_t {
   /** Circular array of the maximum
    * bandwidth-per-NUM_SECS_ROLLING_MEASURE usage for the last
    * NUM_TOTALS periods */
-  int maxima[NUM_TOTALS];
+  uint64_t maxima[NUM_TOTALS];
   /** Circular array of the total bandwidth usage for the last NUM_TOTALS
    * periods */
   uint64_t totals[NUM_TOTALS];
@@ -441,7 +447,7 @@ static INLINE void
 advance_obs(bw_array_t *b)
 {
   int nextidx;
-  int total;
+  uint64_t total;
 
   /* Calculate the total bandwidth for the last NUM_SECS_ROLLING_MEASURE
    * seconds; adjust max_total as needed.*/
@@ -464,7 +470,7 @@ advance_obs(bw_array_t *b)
 /** Add 'n' bytes to the number of bytes in b for second 'when'.
  */
 static INLINE void
-add_obs(bw_array_t *b, time_t when, int n)
+add_obs(bw_array_t *b, time_t when, uint64_t n)
 {
   /* Don't record data in the past. */
   if (when<b->cur_obs_time)
@@ -541,10 +547,11 @@ rep_hist_note_bytes_read(int num_bytes, time_t when)
  * most bandwidth used in any NUM_SECS_ROLLING_MEASURE period for the last
  * NUM_SECS_BW_SUM_IS_VALID seconds.)
  */
-static int
+static uint64_t
 find_largest_max(bw_array_t *b)
 {
-  int i,max;
+  int i;
+  uint64_t max;
   max=0;
   for (i=0; i<NUM_TOTALS; ++i) {
     if (b->maxima[i]>max)
@@ -563,7 +570,7 @@ find_largest_max(bw_array_t *b)
 int
 rep_hist_bandwidth_assess(void)
 {
-  int w,r;
+  uint64_t w,r;
   r = find_largest_max(read_array);
   w = find_largest_max(write_array);
   if (r>w)
@@ -574,6 +581,41 @@ rep_hist_bandwidth_assess(void)
   return 0;
 }
 
+/**
+ * Print the bandwidth history of b (either read_array or write_array)
+ * into the buffer pointed to by buf.  The format is simply comma
+ * separated numbers, from oldest to newest.
+ *
+ * It returns the number of bytes written.
+ */
+static size_t
+rep_hist_fill_bandwidth_history(char *buf, size_t len, bw_array_t *b)
+{
+  char *cp = buf;
+  int i, n;
+
+  if (b->num_maxes_set <= b->next_max_idx) {
+    /* We haven't been through the circular array yet; time starts at i=0.*/
+    i = 0;
+  } else {
+    /* We've been around the array at least once.  The next i to be
+       overwritten is the oldest. */
+    i = b->next_max_idx;
+  }
+
+  for (n=0; n<b->num_maxes_set; ++n,++i) {
+    while (i >= NUM_TOTALS) i -= NUM_TOTALS;
+    if (n==(b->num_maxes_set-1))
+      tor_snprintf(cp, len-(cp-buf), U64_FORMAT,
+                   U64_PRINTF_ARG(b->totals[i]));
+    else
+      tor_snprintf(cp, len-(cp-buf), U64_FORMAT",",
+                   U64_PRINTF_ARG(b->totals[i]));
+    cp += strlen(cp);
+  }
+  return cp-buf;
+}
+
 /**
  * Allocate and return lines for representing this server's bandwidth
  * history in its descriptor.
@@ -583,7 +625,7 @@ rep_hist_get_bandwidth_lines(void)
 {
   char *buf, *cp;
   char t[ISO_TIME_LEN+1];
-  int r, i, n;
+  int r;
   bw_array_t *b;
   size_t len;
 
@@ -599,31 +641,114 @@ rep_hist_get_bandwidth_lines(void)
                  r ? "read-history" : "write-history", t,
                  NUM_SECS_BW_SUM_INTERVAL);
     cp += strlen(cp);
-
-    if (b->num_maxes_set <= b->next_max_idx)
-      /* We haven't been through the circular array yet; time starts at i=0.*/
-      i = 0;
-    else
-      /* We've been around the array at least once.  The next i to be
-         overwritten is the oldest. */
-      i = b->next_max_idx;
-
-    for (n=0; n<b->num_maxes_set; ++n,++i) {
-      while (i >= NUM_TOTALS) i -= NUM_TOTALS;
-      if (n==(b->num_maxes_set-1))
-        tor_snprintf(cp, len-(cp-buf), U64_FORMAT,
-                     U64_PRINTF_ARG(b->totals[i]));
-      else
-        tor_snprintf(cp, len-(cp-buf), U64_FORMAT",",
-                     U64_PRINTF_ARG(b->totals[i]));
-      cp += strlen(cp);
-    }
+    cp += rep_hist_fill_bandwidth_history(cp, len-(cp-buf), b);
     strlcat(cp, "\n", len-(cp-buf));
     ++cp;
   }
   return buf;
 }
 
+/** Update <b>state</b> with the newest bandwidth history. */
+void
+rep_hist_update_state(or_state_t *state)
+{
+  int len, r;
+  char *buf, *cp;
+  smartlist_t **s_values;
+  time_t *s_begins;
+  int *s_interval;
+  bw_array_t *b;
+
+  len = 20*NUM_TOTALS+1;
+  buf = tor_malloc_zero(len);
+
+  for (r=0;r<2;++r) {
+    b = r?read_array:write_array;
+    s_begins  = r?&state->BWHistoryReadEnds    :&state->BWHistoryWriteEnds;
+    s_interval= r?&state->BWHistoryReadInterval:&state->BWHistoryWriteInterval;
+    s_values  = r?&state->BWHistoryReadValues  :&state->BWHistoryWriteValues;
+
+    *s_begins = b->next_period;
+    *s_interval = NUM_SECS_BW_SUM_INTERVAL;
+    if (*s_values) {
+      SMARTLIST_FOREACH(*s_values, char *, cp, tor_free(cp));
+      smartlist_free(*s_values);
+    }
+    cp = buf;
+    cp += rep_hist_fill_bandwidth_history(cp, len, b);
+    tor_snprintf(cp, len-(cp-buf), cp == buf ? U64_FORMAT : ","U64_FORMAT,
+                 U64_PRINTF_ARG(b->total_in_period));
+    *s_values = smartlist_create();
+    if (server_mode(get_options()))
+      smartlist_split_string(*s_values, buf, ",", SPLIT_SKIP_SPACE, 0);
+  }
+  tor_free(buf);
+  state->dirty = 1;
+}
+
+/** Set bandwidth history from our saved state.
+ */
+int
+rep_hist_load_state(or_state_t *state, const char **err)
+{
+  time_t s_begins, start;
+  time_t now = time(NULL);
+  uint64_t v;
+  int r,i,ok;
+  int all_ok = 1;
+  int s_interval;
+  smartlist_t *s_values;
+  bw_array_t *b;
+
+  /* Assert they already have been malloced */
+  tor_assert(read_array && write_array);
+
+  for (r=0;r<2;++r) {
+    b = r?read_array:write_array;
+    s_begins = r?state->BWHistoryReadEnds:state->BWHistoryWriteEnds;
+    s_interval =  r?state->BWHistoryReadInterval:state->BWHistoryWriteInterval;
+    s_values =  r?state->BWHistoryReadValues:state->BWHistoryWriteValues;
+    if (s_values && s_begins >= now - NUM_SECS_BW_SUM_INTERVAL*NUM_TOTALS) {
+      start = s_begins - s_interval*(smartlist_len(s_values));
+
+      b->cur_obs_time = start;
+      b->next_period = start + NUM_SECS_BW_SUM_INTERVAL;
+      SMARTLIST_FOREACH(s_values, char *, cp, {
+        v = tor_parse_uint64(cp, 10, 0, UINT64_MAX, &ok, NULL);
+        if (!ok) {
+          all_ok=0;
+          notice(LD_GENERAL, "Could not parse '%s' into a number.'", cp);
+        }
+        add_obs(b, start, v);
+        start += NUM_SECS_BW_SUM_INTERVAL;
+      });
+    }
+
+    /* Clean up maxima and observed */
+    /* Do we really want to zero this for the purpose of max capacity? */
+    for (i=0; i<NUM_SECS_ROLLING_MEASURE; ++i) {
+      b->obs[i] = 0;
+    }
+    b->total_obs = 0;
+    for (i=0; i<NUM_TOTALS; ++i) {
+      b->maxima[i] = 0;
+    }
+    b->max_total = 0;
+  }
+
+  if (!all_ok) {
+    if (err)
+      *err = "Parsing of bandwidth history values failed";
+    /* and create fresh arrays */
+    tor_free(read_array);
+    tor_free(write_array);
+    read_array = bw_array_new();
+    write_array = bw_array_new();
+    return -1;
+  }
+  return 0;
+}
+
 /** A list of port numbers that have been used recently. */
 static smartlist_t *predicted_ports_list=NULL;
 /** The corresponding most recently used time for each port. */
@@ -766,7 +891,8 @@ rep_hist_note_used_internal(time_t now, int need_uptime, int need_capacity)
 
 /** Return 1 if we've used an internal circ recently; else return 0. */
 int
-rep_hist_get_predicted_internal(time_t now, int *need_uptime, int *need_capacity)
+rep_hist_get_predicted_internal(time_t now, int *need_uptime,
+                                int *need_capacity)
 {
   if (!predicted_internal_time) { /* initialize it */
     predicted_internal_time = now;
