@@ -2,8 +2,9 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: router.c,v 1.234 2005/12/10 09:36:26 arma Exp $ */
-const char router_c_id[] = "$Id: router.c,v 1.234 2005/12/10 09:36:26 arma Exp $";
+/* $Id: router.c,v 1.239 2006/01/10 07:21:01 arma Exp $ */
+const char router_c_id[] =
+  "$Id: router.c,v 1.239 2006/01/10 07:21:01 arma Exp $";
 
 #include "or.h"
 
@@ -241,7 +242,8 @@ init_keys(void)
   char keydir[512];
   char keydir2[512];
   char fingerprint[FINGERPRINT_LEN+1];
-  char fingerprint_line[FINGERPRINT_LEN+MAX_NICKNAME_LEN+3];/*nickname fp\n\0 */
+  /*nickname fp\n\0 */
+  char fingerprint_line[FINGERPRINT_LEN+MAX_NICKNAME_LEN+3];
   char *cp;
   const char *mydesc, *datadir;
   crypto_pk_env_t *prkey;
@@ -465,7 +467,8 @@ router_orport_found_reachable(void)
 {
   if (!can_reach_or_port) {
     if (!clique_mode(get_options()))
-      notice(LD_OR,"Self-testing indicates your ORPort is reachable from the outside. Excellent.%s",
+      notice(LD_OR,"Self-testing indicates your ORPort is reachable from "
+             "the outside. Excellent.%s",
             get_options()->NoPublish ? "" : " Publishing server descriptor.");
     can_reach_or_port = 1;
     mark_my_descriptor_dirty();
@@ -478,7 +481,8 @@ void
 router_dirport_found_reachable(void)
 {
   if (!can_reach_dir_port) {
-    notice(LD_DIRSERV,"Self-testing indicates your DirPort is reachable from the outside. Excellent.");
+    notice(LD_DIRSERV,"Self-testing indicates your DirPort is reachable "
+           "from the outside. Excellent.");
     can_reach_dir_port = 1;
   }
 }
@@ -530,7 +534,8 @@ advertised_server_mode(void)
 }
 
 /**
- * Called with a boolean: set whether we have recently published our descriptor.
+ * Called with a boolean: set whether we have recently published our
+ * descriptor.
  */
 static void
 set_server_advertised(int s)
@@ -574,13 +579,23 @@ decide_if_publishable_server(time_t now)
 
 /** Initiate server descriptor upload as reasonable (if server is publishable,
  * etc).  <b>force</b> is as for router_upload_dir_desc_to_dirservers.
+ *
+ * We need to rebuild the descriptor if it's dirty even if we're not
+ * uploading, because our reachability testing *uses* our descriptor to
+ * determine what IP address and ports to test.
  */
 void
 consider_publishable_server(time_t now, int force)
 {
+  int rebuilt;
+
+  if (!server_mode(get_options()))
+    return;
+
+  rebuilt = router_rebuild_descriptor(0);
   if (decide_if_publishable_server(now)) {
     set_server_advertised(1);
-    if (router_rebuild_descriptor(0) == 0)
+    if (rebuilt == 0)
       router_upload_dir_desc_to_dirservers(force);
   } else {
     set_server_advertised(0);
@@ -593,27 +608,35 @@ consider_publishable_server(time_t now, int force)
 
 /** OR only: if in clique mode, try to open connections to all of the
  * other ORs we know about. Otherwise, open connections to those we
- * think are in clique mode.o
+ * think are in clique mode.
  *
- * If <b>force</b> is zero, only open the connection if we don't already
- * have one.
+ * If <b>testing_reachability</b> is 0, try to open the connections
+ * but only if we don't already have one. If it's 1, then we're an
+ * auth dir server, and we should try to connect regardless of
+ * whether we already have a connection open -- but if <b>try_all</b>
+ * is 0, we want to load balance such that we only try a few connections
+ * per call.
  */
 void
-router_retry_connections(int force)
+router_retry_connections(int testing_reachability, int try_all)
 {
   time_t now = time(NULL);
   routerlist_t *rl = router_get_routerlist();
   or_options_t *options = get_options();
+  static char ctr = 0;
 
   tor_assert(server_mode(options));
 
   SMARTLIST_FOREACH(rl->routers, routerinfo_t *, router, {
+    const char *id_digest = router->cache_info.identity_digest;
     if (router_is_me(router))
       continue;
     if (!clique_mode(options) && !router_is_clique_mode(router))
       continue;
-    if (force ||
-      !connection_or_get_by_identity_digest(router->cache_info.identity_digest)) {
+    if ((testing_reachability &&
+         (try_all || (((uint8_t)id_digest[0]) % 128) == ctr)) ||
+        (!testing_reachability &&
+         !connection_or_get_by_identity_digest(id_digest))) {
       debug(LD_OR,"%sconnecting to %s at %s:%u.",
             clique_mode(options) ? "(forced) " : "",
             router->nickname, router->address, router->or_port);
@@ -621,9 +644,11 @@ router_retry_connections(int force)
       if (!router->testing_since)
         router->testing_since = now;
       connection_or_connect(router->addr, router->or_port,
-                            router->cache_info.identity_digest);
+                            id_digest);
     }
   });
+  if (testing_reachability && !try_all) /* increment ctr */
+    ctr = (ctr + 1) % 128;
 }
 
 /** Return true iff this OR should try to keep connections open to all
@@ -776,7 +801,8 @@ router_rebuild_descriptor(int force)
   ri->or_port = options->ORPort;
   ri->dir_port = options->DirPort;
   ri->cache_info.published_on = time(NULL);
-  ri->onion_pkey = crypto_pk_dup_key(get_onion_key()); /* must invoke from main thread */
+  ri->onion_pkey = crypto_pk_dup_key(get_onion_key()); /* must invoke from
+                                                        * main thread */
   ri->identity_pkey = crypto_pk_dup_key(get_identity_key());
   if (crypto_pk_get_digest(ri->identity_pkey,
                            ri->cache_info.identity_digest)<0) {
@@ -892,7 +918,8 @@ check_descriptor_bandwidth_changed(time_t now)
       cur > prev*2 ||
       cur < prev/2) {
     if (last_changed+MAX_BANDWIDTH_CHANGE_FREQ < now) {
-      info(LD_GENERAL,"Measured bandwidth has changed; rebuilding descriptor.");
+      info(LD_GENERAL,
+           "Measured bandwidth has changed; rebuilding descriptor.");
       mark_my_descriptor_dirty();
       last_changed = now;
     }
@@ -929,7 +956,9 @@ check_descriptor_ipaddress_changed(time_t now)
     in_cur.s_addr = htonl(cur);
     tor_inet_ntoa(&in_cur, addrbuf_cur, sizeof(addrbuf_cur));
 
-    info(LD_GENERAL,"Our IP Address has changed from %s to %s; rebuilding descriptor.", addrbuf_prev, addrbuf_cur);
+    info(LD_GENERAL,
+         "Our IP Address has changed from %s to %s; rebuilding descriptor.",
+         addrbuf_prev, addrbuf_cur);
     mark_my_descriptor_dirty();
   }
 }
@@ -983,7 +1012,8 @@ router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
 
   /* Make sure the identity key matches the one in the routerinfo. */
   if (crypto_pk_cmp_keys(ident_key, router->identity_pkey)) {
-    warn(LD_BUG,"Tried to sign a router with a private key that didn't match router's public key!");
+    warn(LD_BUG,"Tried to sign a router with a private key that didn't "
+         "match router's public key!");
     return -1;
   }
 
@@ -1002,7 +1032,7 @@ router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
 
   /* PEM-encode the identity key key */
   if (crypto_pk_write_public_key_to_string(router->identity_pkey,
-                                           &identity_pkey,&identity_pkeylen)<0) {
+                                        &identity_pkey,&identity_pkeylen)<0) {
     warn(LD_BUG,"write identity_pkey to string failed!");
     tor_free(onion_pkey);
     return -1;
@@ -1101,8 +1131,8 @@ router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
       written += result;
     } else {
       /* There is a range of ports; write ":79-80". */
-      result = tor_snprintf(s+written, maxlen-written, ":%d-%d\n", tmpe->prt_min,
-                        tmpe->prt_max);
+      result = tor_snprintf(s+written, maxlen-written, ":%d-%d\n",
+                            tmpe->prt_min, tmpe->prt_max);
       if (result<0)
         return -1;
       written += result;
@@ -1138,7 +1168,8 @@ router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
   cp = s_tmp = s_dup = tor_strdup(s);
   ri_tmp = router_parse_entry_from_string(cp, NULL);
   if (!ri_tmp) {
-    err(LD_BUG,"We just generated a router descriptor we can't parse: <<%s>>", s);
+    err(LD_BUG,"We just generated a router descriptor we can't parse: <<%s>>",
+        s);
     return -1;
   }
   tor_free(s_dup);
